# -*- coding: utf-8 -*-
# Generated by Django 1.11.17 on 2019-01-04 05:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('replay', '0006_auto_20170225_0018'),
    ]

    operations = [
        migrations.AlterField(
            model_name='action',
            name='data',
            field=models.TextField(help_text='HTTP parameters in JSON format. Supports string templating. Example: $name or $(name).'),
        ),
        migrations.AlterField(
            model_name='action',
            name='files',
            field=models.TextField(help_text='Uploaded files in JSON format. Key and value pairs correspond to filename and file-path pairs. Supports string templating. Example: $name and $(name).'),
        ),
        migrations.AlterField(
            model_name='action',
            name='method',
            field=models.CharField(choices=[('GET', 'GET'), ('POST', 'POST')], max_length=8),
        ),
        migrations.AlterField(
            model_name='action',
            name='name',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='action',
            name='order',
            field=models.FloatField(default=1.0, help_text='Actions are run in ascending order.'),
        ),
        migrations.AlterField(
            model_name='scenario',
            name='priority',
            field=models.FloatField(default=1.0, help_text='Scenarios are run in ascending priority.'),
        ),
        migrations.AlterField(
            model_name='validator',
            name='order',
            field=models.FloatField(default=1.0, help_text='Validators are run in ascending order.'),
        ),
        migrations.AlterField(
            model_name='validator',
            name='pattern',
            field=models.TextField(help_text='Matched against HTTP response. Supports regular expressions. Symbolic group names are stored and may be used for string templating. Example: (?P&lt;name&gt;pattern)'),
        ),
    ]
